package com.mofang.travel.marketing.base.redis;

import cn.hutool.json.JSONUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.data.redis.core.ZSetOperations;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;

@Component
public class RedisClient {

    private final static Logger logger = LoggerFactory.getLogger(RedisClient.class);

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    public void set(String key, String value) {
        try {
            stringRedisTemplate.opsForValue().set(key, value);
        } catch (Exception e) {
            logger.error("redis新增操作失败，key=" + new String(key) + ", " + e.getMessage(), e);
        }
    }

    public void set(String key, int seconds, String value) {
        try {
            stringRedisTemplate.opsForValue().set(key, value, seconds, TimeUnit.SECONDS);
        } catch (Exception e) {
            logger.error("redis新增操作失败，key=" + new String(key) + ", " + e.getMessage(), e);
        }
    }

    public void set(String key, int seconds, Object value) {
        try {
            stringRedisTemplate.opsForValue().set(key, JSONUtil.toJsonStr(value), seconds, TimeUnit.SECONDS);
        } catch (Exception e) {
            logger.error("redis新增操作失败，key=" + new String(key) + ", " + e.getMessage(), e);
        }
    }

    public void set(String key, long seconds, Object value) {
        try {
            stringRedisTemplate.opsForValue().set(key, JSONUtil.toJsonStr(value), seconds, TimeUnit.SECONDS);
        } catch (Exception e) {
            logger.error("redis新增操作失败，key=" + new String(key) + ", " + e.getMessage(), e);
        }
    }

    public void set(String key, String value, long time, TimeUnit timeUnit) {
        if (time <= 0) {
            return;
        }
        try {
            stringRedisTemplate.opsForValue().set(key, value, time, timeUnit);
        } catch (Exception e) {
            logger.error("redis新增操作失败，key=" + new String(key) + ", " + e.getMessage(), e);
        }

    }

    public List<String> multiGet(Collection<String> keys) {
        return stringRedisTemplate.opsForValue().multiGet(keys);
    }

    public <T> T getBean(String key, Class<T> clazz) {
        try {
            String value = stringRedisTemplate.opsForValue().get(key);
            if (StringUtils.isNotBlank(value)) {
                return JSONUtil.toBean(value, clazz);
            }
        } catch (Exception e) {
            logger.error("redis查询操作失败，key=" + key + ", " + e.getMessage(), e);
        }
        return null;
    }

    public String get(String key) {
        try {
            return stringRedisTemplate.opsForValue().get(key);
        } catch (Exception e) {
            logger.error("redis查询操作失败，key=" + new String(key) + ", " + e.getMessage(), e);
        }
        return null;
    }

    public List<String> get(Collection<String> keys) {
        try {
            return stringRedisTemplate.opsForValue().multiGet(keys);
        } catch (Exception e) {
            logger.error("redis查询操作失败，key={}", JSONUtil.toJsonStr(keys), e);
        }
        return null;
    }

    public Boolean del(String key) {
        try {
            return stringRedisTemplate.delete(key);
        } catch (Exception e) {
            logger.error("redis删除操作失败，key=" + new String(key) + ", " + e.getMessage(), e);
        }
        return null;
    }

    /**
     * 批量清除key
     *
     * @param keys
     */
    public void delete(Collection<String> keys) {
        stringRedisTemplate.delete(keys);
    }

    public Object hget(String key, String field) {
        try {
            return redisTemplate.opsForHash().get(key, field);
        } catch (Exception e) {
            logger.error("redis查询操作失败，key=" + key + ", " + e.getMessage(), e);
        }
        return null;
    }

    public void hmset(String key, Map<Object, Object> map) {
        try {
            redisTemplate.opsForHash().putAll(key, map);
        } catch (Exception e) {
            logger.error("redis查询操作失败，key=" + key + ", " + e.getMessage(), e);
        }
    }

    public Long setSize(String key) {
        return redisTemplate.opsForSet().size(key);
    }

    public Boolean setIsMember(String key, Object o) {
        return redisTemplate.opsForSet().isMember(key, o);
    }

    public Set<String> setMembers(String key) {
        try {
            return redisTemplate.opsForSet().members(key);
        } catch (Exception e) {
            logger.error("redis查询操作失败，key=" + key + ", " + e.getMessage(), e);
        }
        return null;
    }

    public void setAdd(String key, String v) {
        try {
            redisTemplate.opsForSet().add(key, v);
        } catch (Exception e) {
            logger.error("redis查询操作失败，key=" + key + ", " + e.getMessage(), e);
        }
    }

    public void setRemove(String key, String v) {
        try {
            redisTemplate.opsForSet().remove(key, v);
        } catch (Exception e) {
            logger.error("redis查询操作失败，key=" + key + ", " + e.getMessage(), e);
        }
    }

    public Boolean getBit(String key, Long offset, boolean defaultValue) {
        try {
            return redisTemplate.opsForValue().getBit(key, offset);
        } catch (Exception e) {
            logger.error("redis getBit操作失败，key=" + key + ", " + e.getMessage(), e);
        }
        return defaultValue;
    }

    public void setBit(String key, Long offset, boolean value) {
        try {
            redisTemplate.opsForValue().setBit(key, offset, value);
        } catch (Exception e) {
            logger.error("redis getBit操作失败，key=" + key + ", " + e.getMessage(), e);
        }
    }

    public boolean zadd(String key, Object value, Object score) {
        try {
            return redisTemplate.opsForZSet().add(key, String.valueOf(value), Double.parseDouble(String.valueOf(score)));
        } catch (Exception e) {
            logger.error("redis zadd 失败，key={},value={},score={} ", key, value, score, e);
        }
        return false;
    }

    public Double score(String key, Object value) {
        return redisTemplate.opsForZSet().score(key, value);
    }

    public double zsetRemove(String key, Object value) {
        return redisTemplate.opsForZSet().remove(key, value);
    }

    public Set<ZSetOperations.TypedTuple<String>> reverseRangeWithScores(String key, long offset, long count) {
        return redisTemplate.opsForZSet().reverseRangeWithScores(key, offset, count);
    }

    public Set<ZSetOperations.TypedTuple<String>> zRangeWithScores(String key, Long startIndex, Long endIndex) {
        try {
            return redisTemplate.opsForZSet().rangeWithScores(key, startIndex, endIndex);
        } catch (Exception e) {
            logger.error(String.format("RedisClient#zRangeWithScores(key,startIndex,endIndex) field,key={%s},startIndex={%d},endIndex={%d}", key, startIndex, endIndex), e);
        }
        return Collections.emptySet();
    }

    public Set<ZSetOperations.TypedTuple<String>> zReverseRangeWithScores(String key, Long startIndex, Long endIndex) {
        try {
            return redisTemplate.opsForZSet().reverseRangeWithScores(key, startIndex, endIndex);
        } catch (Exception e) {
            logger.error(String.format("RedisClient#zRangeWithScores(key,startIndex,endIndex) field,key={%s},startIndex={%d},endIndex={%d}", key, startIndex, endIndex), e);
        }
        return Collections.emptySet();
    }

    public void zRemRangeByScore(String key, Long start, Long end) {
        try {
            redisTemplate.opsForZSet().removeRangeByScore(key, start, end);
        } catch (Exception e) {
            logger.error("redis zRemRangeByScore 失败，key={},start={},end={} ", key, start, end, e);
        }
    }

    public void expire(String key, long timeout) {
        redisTemplate.expire(key, timeout, TimeUnit.SECONDS);
    }

    public void zRem(String key, Object... value) {
        try {
            redisTemplate.opsForZSet().remove(key, value);
        } catch (Exception e) {
            logger.error("redis zRemRangeByScore 失败，key={}", key, e);
        }
    }

    public String incrementValue(String key) {
        ValueOperations<String, String> valueOperations = redisTemplate.opsForValue();
        try {
            valueOperations.increment(key, 1);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return valueOperations.get(key);
    }


    public void putValueExpireTimes(String key, String value, Long timeout) {
        redisTemplate.opsForValue().set(key, value);
        redisTemplate.expire(key, timeout, TimeUnit.SECONDS);
    }

    public boolean setIfAbsent(String key, String value, long time, TimeUnit timeUnit) {
        return redisTemplate.opsForValue().setIfAbsent(key, value);
    }
}
